@@ -23,7 +23,7 @@
 #include <config.h>
 #endif
 
-#include "gdl-i18n.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -37,10 +37,14 @@
  * @title: GdlDockBar
  * @short_description: A docking bar
  * @stability: Unstable
+ * @see_also: #GdlDockMaster
  *
- * A docking bar contains buttons representing minimized dock widgets.
- * The widget can be re-opened by clicking on it. A dock bar is associated
- * to one #GdlDock and will get all minimized widgets of this dock object.
+ * This docking bar is a widget containing a button for each iconified 
+ * #GdlDockItem widget. The widget can be re-opened by clicking on it.
+ * 
+ * A dock bar is associated with one #GdlDockMaster and will get all iconified
+ * widgets of this master. This can includes widgets from several #GdlDock
+ * objects.
  */
 
 
@@ -65,8 +69,8 @@ static void  gdl_dock_bar_set_property    (GObject         *object,
 
 static void  gdl_dock_bar_dispose         (GObject         *object);
 
-static void  gdl_dock_bar_attach          (GdlDockBar      *dockbar,
-                                           GdlDockMaster   *master);
+static void  gdl_dock_bar_set_master      (GdlDockBar      *dockbar,
+                                           GObject         *master);
 static void gdl_dock_bar_remove_item      (GdlDockBar      *dockbar,
                                            GdlDockItem     *item);
 
@@ -123,7 +127,7 @@ gdl_dock_bar_class_init (GdlDockBarClass *klass)
         g_param_spec_object ("master", _("Master"),
                              _("GdlDockMaster object which the dockbar widget "
                                "is attached to"),
-                             GDL_TYPE_DOCK_MASTER,
+                             G_TYPE_OBJECT,
                              G_PARAM_READWRITE));
 
     /**
@@ -185,7 +189,7 @@ gdl_dock_bar_set_property (GObject         *object,
 
     switch (prop_id) {
         case PROP_MASTER:
-            gdl_dock_bar_attach (dockbar, g_value_get_object (value));
+            gdl_dock_bar_set_master (dockbar, g_value_get_object (value));
             break;
         case PROP_DOCKBAR_STYLE:
             dockbar->priv->dockbar_style = g_value_get_enum (value);
@@ -218,7 +222,7 @@ gdl_dock_bar_dispose (GObject *object)
     }
 
     if (priv->master) {
-        gdl_dock_bar_attach (dockbar, NULL);
+        gdl_dock_bar_set_master (dockbar, NULL);
     }
 
    G_OBJECT_CLASS (gdl_dock_bar_parent_class)->dispose (object);
@@ -257,7 +261,6 @@ gdl_dock_bar_item_clicked (GtkWidget   *button,
                            GdlDockItem *item)
 {
     GdlDockBar *dockbar;
-    GdlDockObject *controller;
 
     g_return_if_fail (item != NULL);
 
@@ -265,11 +268,7 @@ gdl_dock_bar_item_clicked (GtkWidget   *button,
     g_assert (dockbar != NULL);
     g_object_set_data (G_OBJECT (item), "GdlDockBar", NULL);
 
-    controller = gdl_dock_master_get_controller (GDL_DOCK_OBJECT_GET_MASTER (item));
-
-    GDL_DOCK_OBJECT_UNSET_FLAGS (item, GDL_DOCK_ICONIFIED);
     gdl_dock_item_show_item (item);
-    gtk_widget_queue_resize (GTK_WIDGET (controller));
 }
 
 static void
@@ -379,10 +378,11 @@ update_dock_items (GdlDockBar *dockbar, gboolean full_update)
             GdlDockItem *item = GDL_DOCK_ITEM (l->data);
 
             if (g_slist_index (dockbar->priv->items, item) != -1 &&
-                !GDL_DOCK_ITEM_ICONIFIED (item))
+                !gdl_dock_item_is_iconified (item))
                 gdl_dock_bar_remove_item (dockbar, item);
             else if (g_slist_index (dockbar->priv->items, item) == -1 &&
-                GDL_DOCK_ITEM_ICONIFIED (item))
+                gdl_dock_item_is_iconified (item) &&
+                !gdl_dock_item_is_placeholder (item))
                 gdl_dock_bar_add_item (dockbar, item);
         }
     } else {
@@ -391,7 +391,8 @@ update_dock_items (GdlDockBar *dockbar, gboolean full_update)
 
             if (g_slist_index (dockbar->priv->items, item) != -1)
                 gdl_dock_bar_remove_item (dockbar, item);
-            if (GDL_DOCK_ITEM_ICONIFIED (item))
+            if (gdl_dock_item_is_iconified (item) &&
+                !gdl_dock_item_is_placeholder (item))
                 gdl_dock_bar_add_item (dockbar, item);
         }
     }
@@ -406,11 +407,11 @@ gdl_dock_bar_layout_changed_cb (GdlDockMaster *master,
 }
 
 static void
-gdl_dock_bar_attach (GdlDockBar    *dockbar,
-                     GdlDockMaster *master)
+gdl_dock_bar_set_master (GdlDockBar    *dockbar,
+                         GObject       *master)
 {
     g_return_if_fail (dockbar != NULL);
-    g_return_if_fail (master == NULL || GDL_IS_DOCK_MASTER (master));
+    g_return_if_fail (master == NULL || GDL_IS_DOCK_MASTER (master) || GDL_IS_DOCK_OBJECT (master));
 
     if (dockbar->priv->master) {
         g_signal_handler_disconnect (dockbar->priv->master,
@@ -418,13 +419,20 @@ gdl_dock_bar_attach (GdlDockBar    *dockbar,
         g_object_unref (dockbar->priv->master);
     }
 
-    dockbar->priv->master = master;
-    if (dockbar->priv->master) {
-        g_object_ref (dockbar->priv->master);
+    if (master != NULL)
+    {
+        /* Accept a GdlDockObject instead of a GdlDockMaster */
+        if (GDL_IS_DOCK_OBJECT (master)) {
+            master = gdl_dock_object_get_master (GDL_DOCK_OBJECT (master));
+        }
+        dockbar->priv->master = g_object_ref (master);
         dockbar->priv->layout_changed_id =
             g_signal_connect (dockbar->priv->master, "layout-changed",
-                              G_CALLBACK (gdl_dock_bar_layout_changed_cb),
+                              (GCallback) gdl_dock_bar_layout_changed_cb,
                               dockbar);
+        
+    } else {
+        dockbar->priv->master = NULL;
     }
 
     update_dock_items (dockbar, FALSE);
@@ -433,20 +441,17 @@ gdl_dock_bar_attach (GdlDockBar    *dockbar,
 
 /**
  * gdl_dock_bar_new:
- * @dock: The associated #GdlDock object
+ * @master: (allow-none): The associated #GdlDockMaster or #GdlDockObject object
  *
- * Creates a new GDL dock bar.
+ * Creates a new GDL dock bar. If a #GdlDockObject is used, the dock bar will
+ * be associated with the master of this object.
  *
  * Returns: The newly created dock bar.
  */
 GtkWidget *
-gdl_dock_bar_new (GdlDock *dock)
+gdl_dock_bar_new (GObject *master)
 {
-    GdlDockMaster *master = NULL;
-
-    /* get the master of the given dock */
-    if (dock)
-        master = GDL_DOCK_OBJECT_GET_MASTER (dock);
+    g_return_val_if_fail (master == NULL || GDL_IS_DOCK_MASTER (master) || GDL_IS_DOCK_OBJECT (master), NULL);
 
     return g_object_new (GDL_TYPE_DOCK_BAR,
                          "master", master, NULL);
@@ -460,8 +465,10 @@ gdl_dock_bar_new (GdlDock *dock)
  * Set the style of the @dockbar.
  */
 void gdl_dock_bar_set_style(GdlDockBar* dockbar,
-			    GdlDockBarStyle style)
+                            GdlDockBarStyle style)
 {
+    g_return_if_fail (GDL_IS_DOCK_BAR (dockbar));
+
     g_object_set(G_OBJECT(dockbar), "dockbar-style", style, NULL);
 }
 
@@ -476,6 +483,10 @@ void gdl_dock_bar_set_style(GdlDockBar* dockbar,
 GdlDockBarStyle gdl_dock_bar_get_style(GdlDockBar* dockbar)
 {
     GdlDockBarStyle style;
+
+    g_return_if_fail (GDL_IS_DOCK_BAR (dockbar));
+
     g_object_get(G_OBJECT(dockbar), "dockbar-style", &style, NULL);
+
     return style;
 }
