@@ -23,14 +23,14 @@
 #include <config.h>
 #endif
 
-#include "gdl-i18n.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <libxml/parser.h>
 #include <gtk/gtk.h>
 
 #include "gdl-dock-layout.h"
-#include "gdl-dock-placeholder.h"
+
 
 /**
  * SECTION:gdl-dock-layout
@@ -76,6 +76,9 @@ enum {
 #define COLUMN_EDITABLE COLUMN_SHOW
 
 struct _GdlDockLayoutPrivate {
+    gboolean              dirty;
+    GdlDockMaster        *master;
+    
     xmlDocPtr         doc;
 
     glong             layout_changed_id;
@@ -123,9 +126,9 @@ gdl_dock_layout_class_init (GdlDockLayoutClass *klass)
     g_object_class_install_property (
         object_class, PROP_MASTER,
         g_param_spec_object ("master", _("Master"),
-                             _("GdlDockMaster object which the layout object "
+                             _("GdlDockMaster or GdlDockObject object which the layout object "
                                "is attached to"),
-                             GDL_TYPE_DOCK_MASTER,
+                             G_TYPE_OBJECT,
                              G_PARAM_READWRITE));
 
     g_object_class_install_property (
@@ -146,10 +149,13 @@ gdl_dock_layout_init (GdlDockLayout *layout)
                                                 GDL_TYPE_DOCK_LAYOUT,
                                                 GdlDockLayoutPrivate);
 
+    layout->priv->master = NULL;
+    layout->priv->dirty = FALSE;
+    layout->priv->idle_save_pending = FALSE;
+#ifndef GDL_DISABLE_DEPRECATED
     layout->master = NULL;
     layout->dirty = FALSE;
-    layout->priv->idle_save_pending = FALSE;
-
+#endif    
 }
 
 static void
@@ -162,7 +168,7 @@ gdl_dock_layout_set_property (GObject      *object,
 
     switch (prop_id) {
         case PROP_MASTER:
-            gdl_dock_layout_attach (layout, g_value_get_object (value));
+            gdl_dock_layout_set_master (layout, g_value_get_object (value));
             break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -179,10 +185,10 @@ gdl_dock_layout_get_property (GObject    *object,
 
     switch (prop_id) {
         case PROP_MASTER:
-            g_value_set_object (value, layout->master);
+            g_value_set_object (value, layout->priv->master);
             break;
         case PROP_DIRTY:
-            g_value_set_boolean (value, layout->dirty);
+            g_value_set_boolean (value, layout->priv->dirty);
             break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -196,8 +202,8 @@ gdl_dock_layout_dispose (GObject *object)
 
     layout = GDL_DOCK_LAYOUT (object);
 
-    if (layout->master)
-        gdl_dock_layout_attach (layout, NULL);
+    if (layout->priv->master)
+        gdl_dock_layout_set_master (layout, NULL);
 
     if (layout->priv->idle_save_pending) {
         layout->priv->idle_save_pending = FALSE;
@@ -361,7 +367,7 @@ gdl_dock_layout_setup_object (GdlDockMaster *master,
            other objects are bound */
         object = g_object_newv (object_type, n_params, params);
         if (object_name) {
-            GDL_DOCK_OBJECT_UNSET_FLAGS (object, GDL_DOCK_AUTOMATIC);
+            gdl_dock_object_set_manual (GDL_DOCK_OBJECT (object));
             gdl_dock_master_add (master, object);
         }
     }
@@ -382,6 +388,7 @@ gdl_dock_layout_setup_object (GdlDockMaster *master,
     /* finally unref object class */
     g_type_class_unref (object_class);
 
+        
     return object;
 }
 
@@ -407,13 +414,23 @@ gdl_dock_layout_recursive_build (GdlDockMaster *master,
         if (object) {
             gdl_dock_object_freeze (object);
 
-            /* recurse here to catch placeholders */
+            /* detach children */
+            if (gdl_dock_object_is_compound (object)) {
+                gtk_container_foreach (GTK_CONTAINER (object),
+                               (GtkCallback) gdl_dock_object_detach,
+                               GINT_TO_POINTER (TRUE));
+            }
+            
+            /* add the object to the parent */
+            if (parent) {
+                if (gdl_dock_object_is_compound (parent)) {
+                    gtk_container_add (GTK_CONTAINER (parent), GTK_WIDGET (object));
+                }
+            }
+
+            /* build children */
             gdl_dock_layout_recursive_build (master, node, object);
-
-            if (GDL_IS_DOCK_PLACEHOLDER (object))
-                /* placeholders are later attached to the parent */
-                gdl_dock_object_detach (object, FALSE);
-
+            
             /* apply "after" parameters */
             for (i = 0; i < n_after_params; i++) {
                 g_object_set_property (G_OBJECT (object),
@@ -424,20 +441,6 @@ gdl_dock_layout_recursive_build (GdlDockMaster *master,
             }
             g_free (after_params);
 
-            /* add the object to the parent */
-            if (parent) {
-                if (GDL_IS_DOCK_PLACEHOLDER (object))
-                    gdl_dock_placeholder_attach (GDL_DOCK_PLACEHOLDER (object),
-                                                 parent);
-                else if (gdl_dock_object_is_compound (parent)) {
-                    gtk_container_add (GTK_CONTAINER (parent), GTK_WIDGET (object));
-                }
-            }
-
-            /* call reduce just in case any child is missing */
-            if (gdl_dock_object_is_compound (object))
-                gdl_dock_object_reduce (object);
-
             gdl_dock_object_thaw (object);
         }
     }
@@ -474,20 +477,16 @@ static void
 gdl_dock_layout_foreach_object_save (GdlDockObject *object,
                                      gpointer       user_data)
 {
-    struct {
-        xmlNodePtr  where;
-        GHashTable *placeholders;
-    } *info = user_data, info_child;
-
+    xmlNodePtr where = (xmlNodePtr)user_data;
     xmlNodePtr   node;
     guint        n_props, i;
     GParamSpec **props;
     GValue       attr = { 0, };
 
     g_return_if_fail (object != NULL && GDL_IS_DOCK_OBJECT (object));
-    g_return_if_fail (info->where != NULL);
+    g_return_if_fail (where != NULL);
 
-    node = xmlNewChild (info->where,
+    node = xmlNewChild (where,
                         NULL,               /* ns */
                         BAD_CAST gdl_dock_object_nick_from_type (G_TYPE_FROM_INSTANCE (object)),
                         BAD_CAST NULL);     /* contents */
@@ -524,45 +523,11 @@ gdl_dock_layout_foreach_object_save (GdlDockObject *object,
     g_value_unset (&attr);
     g_free (props);
 
-    info_child = *info;
-    info_child.where = node;
-
-    /* save placeholders for the object */
-    if (info->placeholders && !GDL_IS_DOCK_PLACEHOLDER (object)) {
-        GList *lph = g_hash_table_lookup (info->placeholders, object);
-        for (; lph; lph = lph->next)
-            gdl_dock_layout_foreach_object_save (GDL_DOCK_OBJECT (lph->data),
-                                                 (gpointer) &info_child);
-    }
-
     /* recurse the object if appropiate */
     if (gdl_dock_object_is_compound (object)) {
         gtk_container_foreach (GTK_CONTAINER (object),
                                (GtkCallback) gdl_dock_layout_foreach_object_save,
-                               (gpointer) &info_child);
-    }
-}
-
-static void
-add_placeholder (GdlDockObject *object,
-                 GHashTable    *placeholders)
-{
-    if (GDL_IS_DOCK_PLACEHOLDER (object)) {
-        GdlDockObject *host;
-        GList *l;
-
-        g_object_get (object, "host", &host, NULL);
-        if (host) {
-            l = g_hash_table_lookup (placeholders, host);
-            /* add the current placeholder to the list of placeholders
-               for that host */
-            if (l)
-                g_hash_table_steal (placeholders, host);
-
-            l = g_list_prepend (l, object);
-            g_hash_table_insert (placeholders, host, l);
-            g_object_unref (host);
-        }
+                               (gpointer) node);
     }
 }
 
@@ -570,32 +535,13 @@ static void
 gdl_dock_layout_save (GdlDockMaster *master,
                       xmlNodePtr     where)
 {
-    struct {
-        xmlNodePtr  where;
-        GHashTable *placeholders;
-    } info;
 
-    GHashTable *placeholders;
-
-    g_return_if_fail (master != NULL && where != NULL);
-
-    /* build the placeholder's hash: the hash keeps lists of
-     * placeholders associated to each object, so that we can save the
-     * placeholders when we are saving the object (since placeholders
-     * don't show up in the normal widget hierarchy) */
-    placeholders = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-                                          NULL, (GDestroyNotify) g_list_free);
-    gdl_dock_master_foreach (master, (GFunc) add_placeholder, placeholders);
+	g_return_if_fail (master != NULL && where != NULL);
 
     /* save the layout recursively */
-    info.where = where;
-    info.placeholders = placeholders;
-
     gdl_dock_master_foreach_toplevel (master, TRUE,
                                       (GFunc) gdl_dock_layout_foreach_object_save,
-                                      (gpointer) &info);
-
-    g_hash_table_destroy (placeholders);
+                                      (gpointer) where);
 }
 
 
@@ -603,20 +549,19 @@ gdl_dock_layout_save (GdlDockMaster *master,
 
 /**
  * gdl_dock_layout_new:
- * @dock: The dock item.
+ * @master: A master or a dock object to which the layout will be attached.
  *
- * Creates a new #GdlDockLayout
+ * Creates a new #GdlDockLayout. Instead of setting @master
+ * directly with a master object, it is possible to use a #GdlDockObject, in
+ * this case the layout will be attached to the same master than the dock
+ * object.
  *
  * Returns: New #GdlDockLayout item.
  */
 GdlDockLayout *
-gdl_dock_layout_new (GdlDock *dock)
+gdl_dock_layout_new (GObject *master)
 {
-    GdlDockMaster *master = NULL;
-
-    /* get the master of the given dock */
-    if (dock)
-        master = GDL_DOCK_OBJECT_GET_MASTER (dock);
+    g_return_val_if_fail (master == NULL || GDL_IS_DOCK_MASTER (master) || GDL_IS_DOCK_OBJECT (master), NULL);
 
     return g_object_new (GDL_TYPE_DOCK_LAYOUT,
                          "master", master,
@@ -646,45 +591,78 @@ gdl_dock_layout_layout_changed_cb (GdlDockMaster *master,
 
 
 /**
- * gdl_dock_layout_attach:
- * @layout: The layout item
- * @master: The master item to which the layout will be attached
+ * gdl_dock_layout_set_master:
+ * @layout: The layout object
+ * @master: The master object to which the layout will be attached
  *
  * Attach the @layout to the @master and delete the reference to
- * the master that the layout attached previously
+ * the master that the layout attached previously. Instead of setting @master
+ * directly with the master object, it is possible to use a #GdlDockObject, in
+ * this case the layout will be attached to the same master than the dock
+ * object.
  */
 void
-gdl_dock_layout_attach (GdlDockLayout *layout,
-                        GdlDockMaster *master)
+gdl_dock_layout_set_master (GdlDockLayout *layout,
+                            GObject *master)
 {
     g_return_if_fail (layout != NULL);
-    g_return_if_fail (master == NULL || GDL_IS_DOCK_MASTER (master));
+    g_return_if_fail (master == NULL || GDL_IS_DOCK_OBJECT (master) || GDL_IS_DOCK_MASTER (master));
 
-    if (layout->master) {
-        g_signal_handler_disconnect (layout->master,
+    if (layout->priv->master) {
+        g_signal_handler_disconnect (layout->priv->master,
                                      layout->priv->layout_changed_id);
-        g_object_unref (layout->master);
+        g_object_unref (layout->priv->master);
     }
 
-    layout->master = master;
-    if (layout->master) {
-        g_object_ref (layout->master);
+    if (master != NULL)
+    {
+        /* Accept a GdlDockObject instead of a GdlDockMaster */
+        if (GDL_IS_DOCK_OBJECT (master)) {
+            master = gdl_dock_object_get_master (GDL_DOCK_OBJECT (master));
+        }
+        layout->priv->master = g_object_ref (master);
         layout->priv->layout_changed_id =
-            g_signal_connect (layout->master, "layout-changed",
+            g_signal_connect (layout->priv->master, "layout-changed",
                               (GCallback) gdl_dock_layout_layout_changed_cb,
                               layout);
+        
+    } else {
+        layout->priv->master = NULL;
     }
+#ifndef GDL_DISABLE_DEPRECATED
+    layout->master = layout->priv->master;
+#endif
 }
 
+/**
+ * gdl_dock_layout_get_master:
+ * @layout: a #GdlDockLayout
+ *
+ * Retrieves the master of the object.
+ *
+ * Return value: (transfer none): a #GdlDockMaster object
+ *
+ * Since: 3.6
+ */
+GObject *
+gdl_dock_layout_get_master (GdlDockLayout *layout)
+{
+    g_return_val_if_fail (GDL_IS_DOCK_LAYOUT (layout), NULL);
+
+    return G_OBJECT (layout->priv->master);
+}
+
+
 /**
 * gdl_dock_layout_load_layout:
 * @layout: The dock item.
 * @name: (allow-none): The name of the layout to load or %NULL for a default layout name.
 *
-* Loads the layout with the given name to the memory.
+* Loads the layout with the given name from the memory.
 * This will set #GdlDockLayout:dirty to %TRUE.
 *
 * See also gdl_dock_layout_load_from_file()
+*
 * Returns: %TRUE if layout successfully loaded else %FALSE
 */
 gboolean
@@ -696,7 +674,7 @@ gdl_dock_layout_load_layout (GdlDockLayout *layout,
 
     g_return_val_if_fail (layout != NULL, FALSE);
 
-    if (!layout->priv->doc || !layout->master)
+    if (!layout->priv->doc || !layout->priv->master)
         return FALSE;
 
     if (!name)
@@ -710,7 +688,7 @@ gdl_dock_layout_load_layout (GdlDockLayout *layout,
         node = gdl_dock_layout_find_layout (layout, NULL);
 
     if (node) {
-        gdl_dock_layout_load (layout->master, node);
+        gdl_dock_layout_load (layout->priv->master, node);
         return TRUE;
     } else
         return FALSE;
@@ -735,7 +713,7 @@ gdl_dock_layout_save_layout (GdlDockLayout *layout,
     gchar      *layout_name;
 
     g_return_if_fail (layout != NULL);
-    g_return_if_fail (layout->master != NULL);
+    g_return_if_fail (layout->priv->master != NULL);
 
     if (!layout->priv->doc)
         gdl_dock_layout_build_doc (layout);
@@ -758,8 +736,8 @@ gdl_dock_layout_save_layout (GdlDockLayout *layout,
     xmlSetProp (node, BAD_CAST NAME_ATTRIBUTE_NAME, BAD_CAST layout_name);
 
     /* save the layout */
-    gdl_dock_layout_save (layout->master, node);
-    layout->dirty = TRUE;
+    gdl_dock_layout_save (layout->priv->master, node);
+    layout->priv->dirty = TRUE;
     g_object_notify (G_OBJECT (layout), "dirty");
 }
 
@@ -787,7 +765,7 @@ gdl_dock_layout_delete_layout (GdlDockLayout *layout,
     if (node) {
         xmlUnlinkNode (node);
         xmlFreeNode (node);
-        layout->dirty = TRUE;
+        layout->priv->dirty = TRUE;
         g_object_notify (G_OBJECT (layout), "dirty");
     }
 }
@@ -811,7 +789,7 @@ gdl_dock_layout_load_from_file (GdlDockLayout *layout,
     if (layout->priv->doc) {
         xmlFreeDoc (layout->priv->doc);
         layout->priv->doc = NULL;
-        layout->dirty = FALSE;
+        layout->priv->dirty = FALSE;
         g_object_notify (G_OBJECT (layout), "dirty");
     }
 
@@ -862,7 +840,7 @@ gdl_dock_layout_save_to_file (GdlDockLayout *layout,
     if (file_handle) {
         bytes = xmlDocDump (file_handle, layout->priv->doc);
         if (bytes >= 0) {
-            layout->dirty = FALSE;
+            layout->priv->dirty = FALSE;
             g_object_notify (G_OBJECT (layout), "dirty");
             retval = TRUE;
         };
@@ -884,7 +862,7 @@ gdl_dock_layout_is_dirty (GdlDockLayout *layout)
 {
     g_return_val_if_fail (layout != NULL, FALSE);
 
-    return layout->dirty;
+    return layout->priv->dirty;
 };
 
 /**
