@@ -25,7 +25,10 @@
 #include <config.h>
 #endif
 
-#include "gdl-i18n.h"
+#ifndef GDL_DISABLE_DEPRECATED
+#define GDL_DISABLE_DEPRECATION_WARNINGS
+
+#include <glib/gi18n-lib.h>
 
 #include "gdl-dock-placeholder.h"
 #include "gdl-dock-item.h"
@@ -43,6 +46,10 @@
  * A dock placeholder is a widget allowing to keep track of a docking place.
  * Unfortunately, all the details of the initial goal have been forgotten and
  * the code has still some issues.
+ * 
+ * In GDL 3.6, this part has been deprecated. Placeholder widgets are not
+ * used anymore. Instead, closed widgets are hidden but kept in the widget
+ * hierarchy.
  */
 
 #undef PLACEHOLDER_DEBUG
@@ -148,7 +155,7 @@ gdl_dock_placeholder_class_init (GdlDockPlaceholderClass *klass)
 
     container_class->add = gdl_dock_placeholder_add;
 
-    dock_object_class->is_compound = FALSE;
+    gdl_dock_object_class_set_is_compound (dock_object_class, FALSE);
     dock_object_class->detach = gdl_dock_placeholder_detach;
     dock_object_class->reduce = gdl_dock_placeholder_reduce;
     dock_object_class->dock = gdl_dock_placeholder_dock;
@@ -509,7 +516,7 @@ gdl_dock_placeholder_dock (GdlDockObject    *object,
         }
         
         /* dock the item as a floating of the controller */
-        toplevel = gdl_dock_master_get_controller (GDL_DOCK_OBJECT_GET_MASTER (ph));
+        toplevel = gdl_dock_object_get_controller (GDL_DOCK_OBJECT (ph));
         gdl_dock_object_dock (toplevel, requestor,
                               GDL_DOCK_FLOATING, NULL);
     }
@@ -562,6 +569,8 @@ gdl_dock_placeholder_present (GdlDockObject *object,
  * working though.
  * 
  * Returns: The newly created placeholder.
+ * 
+ * Deprecated: 3.6:
  */
 GtkWidget * 
 gdl_dock_placeholder_new (const gchar     *name,
@@ -675,7 +684,7 @@ detach_cb (GdlDockObject *object,
 #endif
         /* the toplevel was detached: we attach ourselves to the
            controller with an initial placement of floating */
-        new_host = gdl_dock_master_get_controller (GDL_DOCK_OBJECT_GET_MASTER (ph));
+        new_host = gdl_dock_object_get_controller (GDL_DOCK_OBJECT (ph));
         
         /*
         ph->priv->placement_stack = g_slist_prepend (
@@ -698,6 +707,8 @@ detach_cb (GdlDockObject *object,
  * children and see if any of them matches the placement which is at
  * the top of the stack.  If this is the case, it tries again with the
  * new host.
+ * 
+ * Deprecated: 3.6:
  **/
 static void
 do_excursion (GdlDockPlaceholder *ph)
@@ -826,6 +837,8 @@ connect_host (GdlDockPlaceholder *ph,
  * @object: A new #GdlDockObject
  * 
  * Move the placeholder to the position of @object.
+ * 
+ * Deprecated: 3.6:
  */
 void
 gdl_dock_placeholder_attach (GdlDockPlaceholder *ph,
@@ -853,3 +866,5 @@ gdl_dock_placeholder_attach (GdlDockPlaceholder *ph,
     
     gdl_dock_object_thaw (GDL_DOCK_OBJECT (ph));
 }
+
+#endif
