@@ -24,7 +24,7 @@
 #include <config.h>
 #endif
 
-#include "gdl-i18n.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -32,7 +32,6 @@
 #include "gdl-dock-master.h"
 #include "gdl-dock-paned.h"
 #include "gdl-dock-notebook.h"
-#include "gdl-dock-placeholder.h"
 #include "gdl-preview-window.h"
 
 #include "libgdlmarshal.h"
@@ -45,8 +44,9 @@
  * @stability: Unstable
  *
  * A #GdlDock is the toplevel widget which in turn hold a tree of #GdlDockItem
- * widgets. Each #GdlDock widget is bound to a #GdlDockMaster object.
- * They can exchange widgets between each other if they share the same master.
+ * widgets. 
+ * 
+ * Several dock widgets can exchange widgets if they share the same master. 
  */
 
 #ifndef __FUNCTION__
@@ -130,6 +130,8 @@ static void     gdl_dock_present          (GdlDockObject    *object,
 
 struct _GdlDockPrivate
 {
+    GdlDockObject   *root;
+    
     /* for floating docks */
     gboolean            floating;
     GtkWidget          *window;
@@ -270,8 +272,7 @@ gdl_dock_class_init (GdlDockClass *klass)
     container_class->child_type = gdl_dock_child_type;
     gtk_container_class_handle_border_width (container_class);
 
-    object_class->is_compound = TRUE;
-
+    gdl_dock_object_class_set_is_compound (object_class, TRUE);
     object_class->detach = gdl_dock_detach;
     object_class->reduce = gdl_dock_reduce;
     object_class->dock_request = gdl_dock_dock_request;
@@ -313,7 +314,7 @@ gdl_dock_init (GdlDock *dock)
 
     gtk_widget_set_has_window (GTK_WIDGET (dock), FALSE);
 
-    dock->root = NULL;
+    dock->priv->root = NULL;
     dock->priv->width = -1;
     dock->priv->height = -1;
 }
@@ -351,9 +352,9 @@ gdl_dock_constructor (GType                  type,
         GdlDockMaster *master;
 
         /* create a master for the dock if none was provided in the construction */
-        master = GDL_DOCK_OBJECT_GET_MASTER (GDL_DOCK_OBJECT (dock));
+        master = GDL_DOCK_MASTER (gdl_dock_object_get_master (GDL_DOCK_OBJECT (dock)));
         if (!master) {
-            GDL_DOCK_OBJECT_UNSET_FLAGS (dock, GDL_DOCK_AUTOMATIC);
+            gdl_dock_object_set_manual (GDL_DOCK_OBJECT (dock));
             master = g_object_new (GDL_TYPE_DOCK_MASTER, NULL);
             /* the controller owns the master ref */
             gdl_dock_object_bind (GDL_DOCK_OBJECT (dock), G_OBJECT (master));
@@ -399,7 +400,6 @@ gdl_dock_constructor (GType                  type,
                               G_CALLBACK (gdl_dock_floating_window_delete_event_cb),
                               NULL);
         }
-        GDL_DOCK_OBJECT_SET_FLAGS (dock, GDL_DOCK_ATTACHED);
     }
 
     return g_object;
@@ -418,8 +418,8 @@ gdl_dock_set_property  (GObject      *object,
             dock->priv->floating = g_value_get_boolean (value);
             break;
         case PROP_DEFAULT_TITLE:
-            if (GDL_DOCK_OBJECT (object)->master)
-                g_object_set (GDL_DOCK_OBJECT (object)->master,
+            if (gdl_dock_object_get_master (GDL_DOCK_OBJECT (object)) != NULL)
+                g_object_set (gdl_dock_object_get_master (GDL_DOCK_OBJECT (object)),
                               "default-title", g_value_get_string (value),
                               NULL);
             break;
@@ -470,9 +470,9 @@ gdl_dock_get_property  (GObject      *object,
             g_value_set_boolean (value, dock->priv->floating);
             break;
         case PROP_DEFAULT_TITLE:
-            if (GDL_DOCK_OBJECT (object)->master) {
+            if (gdl_dock_object_get_master (GDL_DOCK_OBJECT (object)) != NULL) {
                 gchar *default_title;
-                g_object_get (GDL_DOCK_OBJECT (object)->master,
+                g_object_get (gdl_dock_object_get_master (GDL_DOCK_OBJECT (object)),
                               "default-title", &default_title,
                               NULL);
 
@@ -531,22 +531,21 @@ gdl_dock_set_title (GdlDock *dock)
     if (!dock->priv->window)
         return;
 
-    if (!dock->priv->auto_title && object->long_name) {
-        title = object->long_name;
+    if (!dock->priv->auto_title && gdl_dock_object_get_long_name (object)) {
+        title = g_strdup (gdl_dock_object_get_long_name (object));
     }
-    else if (object->master) {
-        g_object_get (object->master, "default-title", &title, NULL);
+    else if (gdl_dock_object_get_master (object) != NULL) {
+        g_object_get (G_OBJECT (gdl_dock_object_get_master (object)), "default-title", &title, NULL);
     }
 
-    if (!title && dock->root) {
-        g_object_get (dock->root, "long-name", &title, NULL);
+    if (!title && dock->priv->root) {
+        g_object_get (dock->priv->root, "long-name", &title, NULL);
     }
 
     if (!title) {
         /* set a default title in the long_name */
         dock->priv->auto_title = TRUE;
-        title = g_strdup_printf (
-            _("Dock #%d"), GDL_DOCK_MASTER (object->master)->dock_number++);
+        title = gdl_dock_master_get_dock_name (GDL_DOCK_MASTER (gdl_dock_object_get_master (object)));
     }
 
     gtk_window_set_title (GTK_WINDOW (dock->priv->window), title);
@@ -613,12 +612,12 @@ gdl_dock_get_size (GtkWidget     *widget,
     *minimum = *natural = 0;
 
     /* make request to root */
-    if (dock->root && gtk_widget_get_visible (GTK_WIDGET (dock->root))) {
+    if (dock->priv->root && gtk_widget_get_visible (GTK_WIDGET (dock->priv->root))) {
         if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-            gtk_widget_get_preferred_width (GTK_WIDGET (dock->root), minimum, natural);
+            gtk_widget_get_preferred_width (GTK_WIDGET (dock->priv->root), minimum, natural);
         }
         else {
-            gtk_widget_get_preferred_height (GTK_WIDGET (dock->root), minimum, natural);
+            gtk_widget_get_preferred_height (GTK_WIDGET (dock->priv->root), minimum, natural);
         }
     }
 }
@@ -654,8 +653,8 @@ gdl_dock_size_allocate (GtkWidget     *widget,
 
     gtk_widget_set_allocation (widget, allocation);
 
-    if (dock->root && gtk_widget_get_visible (GTK_WIDGET (dock->root)))
-        gtk_widget_size_allocate (GTK_WIDGET (dock->root), allocation);
+    if (dock->priv->root && gtk_widget_get_visible (GTK_WIDGET (dock->priv->root)))
+        gtk_widget_size_allocate (GTK_WIDGET (dock->priv->root), allocation);
 }
 
 static void
@@ -671,8 +670,8 @@ gdl_dock_map (GtkWidget *widget)
 
     GTK_WIDGET_CLASS (gdl_dock_parent_class)->map (widget);
 
-    if (dock->root) {
-        child = GTK_WIDGET (dock->root);
+    if (dock->priv->root) {
+        child = GTK_WIDGET (dock->priv->root);
         if (gtk_widget_get_visible (child) && !gtk_widget_get_mapped (child))
             gtk_widget_map (child);
     }
@@ -691,8 +690,8 @@ gdl_dock_unmap (GtkWidget *widget)
 
     GTK_WIDGET_CLASS (gdl_dock_parent_class)->unmap (widget);
 
-    if (dock->root) {
-        child = GTK_WIDGET (dock->root);
+    if (dock->priv->root) {
+        child = GTK_WIDGET (dock->priv->root);
         if (gtk_widget_get_visible (child) && gtk_widget_get_mapped (child))
             gtk_widget_unmap (child);
     }
@@ -707,7 +706,7 @@ gdl_dock_foreach_automatic (GdlDockObject *object,
 {
     void (* function) (GtkWidget *) = user_data;
 
-    if (GDL_DOCK_OBJECT_AUTOMATIC (object))
+    if (gdl_dock_object_is_automatic (object))
         (* function) (GTK_WIDGET (object));
 }
 
@@ -715,6 +714,7 @@ static void
 gdl_dock_show (GtkWidget *widget)
 {
     GdlDock *dock;
+    GdlDockMaster *master;
 
     g_return_if_fail (widget != NULL);
     g_return_if_fail (GDL_IS_DOCK (widget));
@@ -725,8 +725,9 @@ gdl_dock_show (GtkWidget *widget)
     if (dock->priv->floating && dock->priv->window)
         gtk_widget_show (dock->priv->window);
 
-    if (GDL_DOCK_IS_CONTROLLER (dock)) {
-        gdl_dock_master_foreach_toplevel (GDL_DOCK_OBJECT_GET_MASTER (dock),
+    master = GDL_DOCK_MASTER (gdl_dock_object_get_master (GDL_DOCK_OBJECT (dock)));
+    if (GDL_DOCK (gdl_dock_master_get_controller (master)) == dock) {
+        gdl_dock_master_foreach_toplevel (master,
                                           FALSE, (GFunc) gdl_dock_foreach_automatic,
                                           gtk_widget_show);
     }
@@ -736,6 +737,7 @@ static void
 gdl_dock_hide (GtkWidget *widget)
 {
     GdlDock *dock;
+    GdlDockMaster *master;
 
     g_return_if_fail (widget != NULL);
     g_return_if_fail (GDL_IS_DOCK (widget));
@@ -746,8 +748,9 @@ gdl_dock_hide (GtkWidget *widget)
     if (dock->priv->floating && dock->priv->window)
         gtk_widget_hide (dock->priv->window);
 
-    if (GDL_DOCK_IS_CONTROLLER (dock)) {
-        gdl_dock_master_foreach_toplevel (GDL_DOCK_OBJECT_GET_MASTER (dock),
+    master = GDL_DOCK_MASTER (gdl_dock_object_get_master (GDL_DOCK_OBJECT (dock)));
+    if (GDL_DOCK (gdl_dock_master_get_controller (master)) == dock) {
+        gdl_dock_master_foreach_toplevel (master,
                                           FALSE, (GFunc) gdl_dock_foreach_automatic,
                                           gtk_widget_hide);
     }
@@ -779,9 +782,8 @@ gdl_dock_remove (GtkContainer *container,
     dock = GDL_DOCK (container);
     was_visible = gtk_widget_get_visible (widget);
 
-    if (GTK_WIDGET (dock->root) == widget) {
-        dock->root = NULL;
-        GDL_DOCK_OBJECT_UNSET_FLAGS (widget, GDL_DOCK_ATTACHED);
+    if (GTK_WIDGET (dock->priv->root) == widget) {
+        dock->priv->root = NULL;
         gtk_widget_unparent (widget);
 
         if (was_visible && gtk_widget_get_visible (GTK_WIDGET (container)))
@@ -803,8 +805,8 @@ gdl_dock_forall (GtkContainer *container,
 
     dock = GDL_DOCK (container);
 
-    if (dock->root)
-        (*callback) (GTK_WIDGET (dock->root), callback_data);
+    if (dock->priv->root)
+        (*callback) (GTK_WIDGET (dock->priv->root), callback_data);
 }
 
 static GType
@@ -820,10 +822,9 @@ gdl_dock_detach (GdlDockObject *object,
     GdlDock *dock = GDL_DOCK (object);
 
     /* detach children */
-    if (recursive && dock->root) {
-        gdl_dock_object_detach (dock->root, recursive);
+    if (recursive && dock->priv->root) {
+        gdl_dock_object_detach (dock->priv->root, recursive);
     }
-    GDL_DOCK_OBJECT_UNSET_FLAGS (object, GDL_DOCK_ATTACHED);
 }
 
 static void
@@ -832,13 +833,13 @@ gdl_dock_reduce (GdlDockObject *object)
     GdlDock *dock = GDL_DOCK (object);
     GtkWidget *parent;
 
-    if (dock->root)
+    if (dock->priv->root)
         return;
 
-    if (GDL_DOCK_OBJECT_AUTOMATIC (dock)) {
+    if (gdl_dock_object_is_automatic (GDL_DOCK_OBJECT (dock))) {
         gtk_widget_destroy (GTK_WIDGET (dock));
 
-    } else if (!GDL_DOCK_OBJECT_ATTACHED (dock)) {
+    } else if (gdl_dock_object_is_closed (GDL_DOCK_OBJECT (dock))) {
         /* if the user explicitly detached the object */
         if (dock->priv->floating)
             gtk_widget_hide (GTK_WIDGET (dock));
@@ -896,11 +897,11 @@ gdl_dock_dock_request (GdlDockObject  *object,
 
 	/* If GdlDock has no root item yet, set the dock itself as
 	   possible target. */
-        if (!dock->root) {
+        if (!dock->priv->root) {
             my_request.position = GDL_DOCK_TOP;
             my_request.target = object;
         } else {
-            my_request.target = dock->root;
+            my_request.target = dock->priv->root;
 
             /* See if it's in the border_width band. */
             if (rel_x < bw) {
@@ -921,7 +922,7 @@ gdl_dock_dock_request (GdlDockObject  *object,
                 /* Otherwise try our children. */
                 /* give them allocation coordinates (we are a
                    GTK_NO_WINDOW) widget */
-                may_dock = gdl_dock_object_dock_request (GDL_DOCK_OBJECT (dock->root),
+                may_dock = gdl_dock_object_dock_request (GDL_DOCK_OBJECT (dock->priv->root),
                                                          x, y, &my_request);
             }
         }
@@ -968,18 +969,17 @@ gdl_dock_dock (GdlDockObject    *object,
         gdl_dock_add_floating_item (dock, item,
                                     x, y, width, height);
     }
-    else if (dock->root) {
+    else if (dock->priv->root) {
         /* This is somewhat a special case since we know which item to
            pass the request on because we only have on child */
-        gdl_dock_object_dock (dock->root, requestor, position, NULL);
+        gdl_dock_object_dock (dock->priv->root, requestor, position, NULL);
         gdl_dock_set_title (dock);
 
     }
     else { /* Item about to be added is root item. */
         GtkWidget *widget = GTK_WIDGET (requestor);
 
-        dock->root = requestor;
-        GDL_DOCK_OBJECT_SET_FLAGS (requestor, GDL_DOCK_ATTACHED);
+        dock->priv->root = requestor;
         gtk_widget_set_parent (widget, GTK_WIDGET (dock));
 
         gdl_dock_item_show_grip (GDL_DOCK_ITEM (requestor));
@@ -1012,9 +1012,9 @@ gdl_dock_floating_window_delete_event_cb (GtkWidget *widget)
     g_return_val_if_fail (GTK_IS_WINDOW (widget), FALSE);
 
     dock = GDL_DOCK (g_object_get_data (G_OBJECT (widget), "dock"));
-    if (dock->root) {
+    if (dock->priv->root) {
         /* this will call reduce on ourselves, hiding the window if appropiate */
-        gdl_dock_item_hide_item (GDL_DOCK_ITEM (dock->root));
+        gdl_dock_item_hide_item (GDL_DOCK_ITEM (dock->priv->root));
     }
 
     return TRUE;
@@ -1041,7 +1041,7 @@ gdl_dock_reorder (GdlDockObject    *object,
 
     if (dock->priv->floating &&
         new_position == GDL_DOCK_FLOATING &&
-        dock->root == requestor) {
+        dock->priv->root == requestor) {
 
         if (other_data && G_VALUE_HOLDS (other_data, GDK_TYPE_RECTANGLE)) {
             GdkRectangle *rect;
@@ -1065,7 +1065,7 @@ gdl_dock_child_placement (GdlDockObject    *object,
     GdlDock *dock = GDL_DOCK (object);
     gboolean retval = TRUE;
 
-    if (dock->root == child) {
+    if (dock->priv->root == child) {
         if (placement) {
             if (*placement == GDL_DOCK_NONE || *placement == GDL_DOCK_FLOATING)
                 *placement = GDL_DOCK_TOP;
@@ -1102,7 +1102,7 @@ gdl_dock_new (void)
     GObject *dock;
 
     dock = g_object_new (GDL_TYPE_DOCK, NULL);
-    GDL_DOCK_OBJECT_UNSET_FLAGS (dock, GDL_DOCK_AUTOMATIC);
+    gdl_dock_object_set_manual (GDL_DOCK_OBJECT (dock));
 
     return GTK_WIDGET (dock);
 }
@@ -1125,10 +1125,10 @@ gdl_dock_new_from (GdlDock  *original,
     g_return_val_if_fail (original != NULL, NULL);
 
     new_dock = g_object_new (GDL_TYPE_DOCK,
-                             "master", GDL_DOCK_OBJECT_GET_MASTER (original),
+                             "master", gdl_dock_object_get_master (GDL_DOCK_OBJECT (original)),
                              "floating", floating,
                              NULL);
-    GDL_DOCK_OBJECT_UNSET_FLAGS (new_dock, GDL_DOCK_AUTOMATIC);
+    gdl_dock_object_set_manual (GDL_DOCK_OBJECT (new_dock));
 
     return GTK_WIDGET (new_dock);
 }
@@ -1307,11 +1307,16 @@ gdl_dock_add_item (GdlDock          *dock,
     g_return_if_fail (dock != NULL);
     g_return_if_fail (item != NULL);
 
-    /* Check if a placeholder widget already exist */
-    placeholder = gdl_dock_master_get_object (GDL_DOCK_OBJECT_GET_MASTER (dock), GDL_DOCK_OBJECT (item)->name);
-    if ((placeholder != NULL))
-        parent = gdl_dock_object_get_parent_object (placeholder);
-
+    /* Check if a placeholder widget already exist in the same dock */
+    placeholder = gdl_dock_master_get_object (GDL_DOCK_MASTER (gdl_dock_object_get_master (GDL_DOCK_OBJECT (dock))), gdl_dock_object_get_name (GDL_DOCK_OBJECT (item)));
+    if ((placeholder != GDL_DOCK_OBJECT (item)) && (placeholder != NULL)) {
+        if (gdl_dock_object_get_toplevel (placeholder) == dock) {
+            parent = gdl_dock_object_get_parent_object (placeholder);
+        } else {
+            gtk_widget_destroy (GTK_WIDGET (placeholder));
+        }
+    }
+    
     if (parent && gdl_dock_object_child_placement (parent, placeholder, &place))
     {
         gdl_dock_object_freeze (GDL_DOCK_OBJECT (parent));
@@ -1329,12 +1334,12 @@ gdl_dock_add_item (GdlDock          *dock,
     else {
         GdlDockItem *best_dock_item;
         /* Non-floating item. */
-        if (dock->root) {
+        if (dock->priv->root) {
             GdlDockPlacement local_placement;
             GtkRequisition preferred_size;
 
             best_dock_item =
-                gdl_dock_find_best_placement_item (GDL_DOCK_ITEM (dock->root),
+                gdl_dock_find_best_placement_item (GDL_DOCK_ITEM (dock->priv->root),
                                                    placement, 0);
             local_placement = gdl_dock_refine_placement (dock, best_dock_item,
                                                          placement);
@@ -1375,7 +1380,7 @@ gdl_dock_add_floating_item (GdlDock        *dock,
     g_return_if_fail (item != NULL);
 
     new_dock = GDL_DOCK (g_object_new (GDL_TYPE_DOCK,
-                                       "master", GDL_DOCK_OBJECT_GET_MASTER (dock),
+                                       "master", gdl_dock_object_get_master (GDL_DOCK_OBJECT (dock)),
                                        "floating", TRUE,
                                        "width", width,
                                        "height", height,
@@ -1415,36 +1420,11 @@ gdl_dock_get_item_by_name (GdlDock     *dock,
     g_return_val_if_fail (dock != NULL && name != NULL, NULL);
 
     /* proxy the call to our master */
-    found = gdl_dock_master_get_object (GDL_DOCK_OBJECT_GET_MASTER (dock), name);
+    found = gdl_dock_master_get_object (GDL_DOCK_MASTER (gdl_dock_object_get_master (GDL_DOCK_OBJECT (dock))), name);
 
     return (found && GDL_IS_DOCK_ITEM (found)) ? GDL_DOCK_ITEM (found) : NULL;
 }
 
-/**
- * gdl_dock_get_placeholder_by_name:
- * @dock: A #GdlDock widget
- * @name: An item name
- *
- * Looks for an #GdlDockPlaceholder object bound to the master of the dock item.
- * It does not search only in the children of this particular dock widget.
- *
- * Returns: (transfer none): A #GdlDockPlaceholder object or %NULL
- */
-GdlDockPlaceholder *
-gdl_dock_get_placeholder_by_name (GdlDock     *dock,
-                                  const gchar *name)
-{
-    GdlDockObject *found;
-
-    g_return_val_if_fail (dock != NULL && name != NULL, NULL);
-
-    /* proxy the call to our master */
-    found = gdl_dock_master_get_object (GDL_DOCK_OBJECT_GET_MASTER (dock), name);
-
-    return (found && GDL_IS_DOCK_PLACEHOLDER (found)) ?
-        GDL_DOCK_PLACEHOLDER (found) : NULL;
-}
-
 /**
  * gdl_dock_get_named_items:
  * @dock: A #GdlDock widget
@@ -1462,7 +1442,7 @@ gdl_dock_get_named_items (GdlDock *dock)
 
     g_return_val_if_fail (dock != NULL, NULL);
 
-    gdl_dock_master_foreach (GDL_DOCK_OBJECT_GET_MASTER (dock),
+    gdl_dock_master_foreach (GDL_DOCK_MASTER (gdl_dock_object_get_master (GDL_DOCK_OBJECT (dock))),
                              (GFunc) _gdl_dock_foreach_build_list, &list);
 
     return list;
@@ -1489,6 +1469,23 @@ gdl_dock_object_get_toplevel (GdlDockObject *object)
     return parent ? GDL_DOCK (parent) : NULL;
 }
 
+
+/**
+ * gdl_dock_get_root:
+ * @object: A #GdlDockObject
+ *
+ * Get the first child of the #GdlDockObject.
+ *
+ * Returns: (allow-none) (transfer none): A #GdlDockObject or %NULL.
+ */
+GdlDockObject *
+gdl_dock_get_root (GdlDock *dock)
+{
+    g_return_val_if_fail (GDL_IS_DOCK (dock), NULL);
+    
+    return dock->priv->root;
+}
+
 /**
  * gdl_dock_show_preview:
  * @dock: A #GdlDock widget
