@@ -30,7 +30,7 @@
 #include <config.h>
 #endif
 
-#include "gdl-i18n.h"
+#include <glib/gi18n-lib.h>
 #include "gdl-switcher.h"
 #include "libgdlmarshal.h"
 #include "libgdltypebuiltins.h"
@@ -1159,7 +1159,7 @@ gdl_switcher_set_show_buttons (GdlSwitcher *switcher, gboolean show)
     for (p = switcher->priv->buttons; p != NULL; p = p->next) {
         Button *button = p->data;
 
-        if (show)
+        if (show && gtk_widget_get_visible (button->page))
             gtk_widget_show (button->button_widget);
         else
             gtk_widget_hide (button->button_widget);
